@@ -1,7 +1,7 @@
-# encoding: utf-8
+# -*- coding: utf-8 -*-
+# pylint: disable=too-many-lines
 """Module containing :class:`Terminal`, the primary API entry point."""
 # std imports
-import io
 import os
 import re
 import sys
@@ -27,10 +27,12 @@ from .keyboard import (_time_left,
 from .sequences import Termcap, Sequence, SequenceTextWrapper
 from .colorspace import RGB_256TABLE
 from .formatters import (COLORS,
+                         COMPOUNDABLES,
                          FormattingString,
-                         FormattingOtherString,
                          NullCallableString,
                          ParameterizingString,
+                         FormattingOtherString,
+                         split_compound,
                          resolve_attribute,
                          resolve_capability)
 from ._capabilities import CAPABILITY_DATABASE, CAPABILITIES_ADDITIVES, CAPABILITIES_RAW_MIXIN
@@ -42,15 +44,6 @@ except NameError:
     # pylint: disable=redefined-builtin
     InterruptedError = select.error
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    # python 2.6 requires 3rd party library (backport)
-    #
-    # pylint: disable=import-error
-    #         Unable to import 'ordereddict'
-    from ordereddict import OrderedDict
-
 
 HAS_TTY = True
 if platform.system() == 'Windows':
@@ -155,56 +148,41 @@ class Terminal(object):
         """
         # pylint: disable=global-statement,too-many-branches
         global _CUR_TERM
-        self._keyboard_fd = None
-
-        # Default stream is stdout, keyboard valid as stdin only when
-        # output stream is stdout or stderr and is a tty.
-        if stream is None:
-            stream = sys.__stdout__
-        if stream in (sys.__stdout__, sys.__stderr__):
-            self._keyboard_fd = sys.__stdin__.fileno()
-
+        self.errors = ['parameters: kind=%r, stream=%r, force_styling=%r' %
+                       (kind, stream, force_styling)]
+        self._normal = None  # cache normal attr, preventing recursive lookups
         # we assume our input stream to be line-buffered until either the
-        # cbreak of raw context manager methods are entered with an
-        # attached tty.
+        # cbreak of raw context manager methods are entered with an attached tty.
         self._line_buffered = True
 
-        try:
-            stream_fd = (stream.fileno() if hasattr(stream, 'fileno') and
-                         callable(stream.fileno) else None)
-        except (io.UnsupportedOperation, ValueError):
-            # The stream is not a file, such as the case of StringIO, or, when it has been
-            # "detached", such as might be the case of stdout in some test scenarios.
-            stream_fd = None
-
         self._stream = stream
-        self._is_a_tty = stream_fd is not None and os.isatty(stream_fd)
-        self._does_styling = ((self.is_a_tty or force_styling) and
-                              force_styling is not None)
+        self._keyboard_fd = None
+        self._init_descriptor = None
+        self._is_a_tty = False
+        self.__init__streams()
 
-        # _keyboard_fd only non-None if both stdin and stdout is a tty.
-        self._keyboard_fd = (self._keyboard_fd
-                             if self._keyboard_fd is not None and
-                             self.is_a_tty and os.isatty(self._keyboard_fd)
-                             else None)
-        self._normal = None  # cache normal attr, preventing recursive lookups
-
-        # The descriptor to direct terminal initialization sequences to.
-        self._init_descriptor = (sys.__stdout__.fileno() if stream_fd is None
-                                 else stream_fd)
-
-        if platform.system() == 'Windows':
+        if platform.system() == 'Windows' and self._init_descriptor is not None:
             self._kind = kind or curses.get_term(self._init_descriptor)
         else:
             self._kind = kind or os.environ.get('TERM', 'dumb') or 'dumb'
 
+        self._does_styling = False
+        if force_styling:
+            self._does_styling = True
+        elif self.is_a_tty:
+            if force_styling is None:
+                self.errors.append('force_styling is None')
+            else:
+                self._does_styling = True
+
         if self.does_styling:
             # Initialize curses (call setupterm), so things like tigetstr() work.
             try:
-                curses.setupterm(self._kind, open(os.devnull).fileno())
+                curses.setupterm(self._kind, self._init_descriptor)
             except curses.error as err:
-                warnings.warn('Failed to setupterm(kind={0!r}): {1}'
-                              .format(self._kind, err))
+                msg = 'Failed to setupterm(kind={0!r}): {1}'.format(self._kind, err)
+                warnings.warn(msg)
+                self.errors.append(msg)
                 self._kind = None
                 self._does_styling = False
             else:
@@ -227,13 +205,62 @@ class Terminal(object):
         self.__init__capabilities()
         self.__init__keycodes()
 
+    def __init__streams(self):
+        # pylint: disable=too-complex,too-many-branches
+        #         Agree to disagree !
+        stream_fd = None
+
+        # Default stream is stdout
+        if self._stream is None:
+            self._stream = sys.__stdout__
+
+        if not hasattr(self._stream, 'fileno'):
+            self.errors.append('stream has no fileno method')
+        elif not callable(self._stream.fileno):
+            self.errors.append('stream.fileno is not callable')
+        else:
+            try:
+                stream_fd = self._stream.fileno()
+            except ValueError as err:
+                # The stream is not a file, such as the case of StringIO, or, when it has been
+                # "detached", such as might be the case of stdout in some test scenarios.
+                self.errors.append('Unable to determine output stream file descriptor: %s' % err)
+            else:
+                self._is_a_tty = os.isatty(stream_fd)
+                if not self._is_a_tty:
+                    self.errors.append('stream not a TTY')
+
+        # Keyboard valid as stdin only when output stream is stdout or stderr and is a tty.
+        if self._stream in (sys.__stdout__, sys.__stderr__):
+            try:
+                self._keyboard_fd = sys.__stdin__.fileno()
+            except (AttributeError, ValueError) as err:
+                self.errors.append('Unable to determine input stream file descriptor: %s' % err)
+            else:
+                # _keyboard_fd only non-None if both stdin and stdout is a tty.
+                if not self.is_a_tty:
+                    self.errors.append('Output stream is not a TTY')
+                    self._keyboard_fd = None
+                elif not os.isatty(self._keyboard_fd):
+                    self.errors.append('Input stream is not a TTY')
+                    self._keyboard_fd = None
+        else:
+            self.errors.append('Output stream is not a default stream')
+
+        # The descriptor to direct terminal initialization sequences to.
+        self._init_descriptor = stream_fd
+        if stream_fd is None:
+            try:
+                self._init_descriptor = sys.__stdout__.fileno()
+            except ValueError as err:
+                self.errors.append('Unable to determine __stdout__ file descriptor: %s' % err)
+
     def __init__color_capabilities(self):
         self._color_distance_algorithm = 'cie2000'
         if not self.does_styling:
             self.number_of_colors = 0
         elif platform.system() == 'Windows' or (
-            os.environ.get('COLORTERM') in ('truecolor', '24bit')
-        ):
+                os.environ.get('COLORTERM') in ('truecolor', '24bit')):
             self.number_of_colors = 1 << 24
         else:
             self.number_of_colors = max(0, curses.tigetnum('colors') or -1)
@@ -245,7 +272,7 @@ class Terminal(object):
     def __init__capabilities(self):
         # important that we lay these in their ordered direction, so that our
         # preferred, 'color' over 'set_a_attributes1', for example.
-        self.caps = OrderedDict()
+        self.caps = collections.OrderedDict()
 
         # some static injected patterns, esp. without named attribute access.
         for name, (attribute, pattern) in CAPABILITIES_ADDITIVES.items():
@@ -298,24 +325,14 @@ class Terminal(object):
 
         if self._keyboard_fd is not None:
             # set input encoding and initialize incremental decoder
-            if platform.system() == 'Windows' and sys.version_info[0] < 3:
-                # Default for setlocale() has side effects for PY2 on Windows
-                pass
-            else:
-                locale.setlocale(locale.LC_ALL, '')
-            self._encoding = locale.getpreferredencoding() or 'ascii'
-
+            self._encoding = locale.getpreferredencoding() or 'UTF-8'
             try:
-                self._keyboard_decoder = codecs.getincrementaldecoder(
-                    self._encoding)()
-
+                self._keyboard_decoder = codecs.getincrementaldecoder(self._encoding)()
             except LookupError as err:
-                # encoding is illegal or unsupported, use 'ascii'
-                warnings.warn('LookupError: {0}, fallback to ASCII for '
-                              'keyboard.'.format(err))
-                self._encoding = 'ascii'
-                self._keyboard_decoder = codecs.getincrementaldecoder(
-                    self._encoding)()
+                # encoding is illegal or unsupported, use 'UTF-8'
+                warnings.warn('LookupError: {0}, defaulting to UTF-8 for keyboard.'.format(err))
+                self._encoding = 'UTF-8'
+                self._keyboard_decoder = codecs.getincrementaldecoder(self._encoding)()
 
     def __getattr__(self, attr):
         r"""
@@ -337,10 +354,10 @@ class Terminal(object):
             >>> term.move(line, column)
 
         See the manual page `terminfo(5)
-        <http://invisible-island.net/ncurses/man/terminfo.5.html>`_ for a
+        <https://invisible-island.net/ncurses/man/terminfo.5.html>`_ for a
         complete list of capabilities and their arguments.
         """
-        if not self.does_styling:
+        if not self._does_styling:
             return NullCallableString()
         # Fetch the missing 'attribute' into some kind of curses-resolved
         # capability, and cache by attaching to this Terminal class instance.
@@ -396,6 +413,24 @@ class Terminal(object):
         """
         return self._height_and_width().ws_col
 
+    @property
+    def pixel_height(self):
+        """
+        Read-only property: Height ofthe terminal (in pixels).
+
+        :rtype: int
+        """
+        return self._height_and_width().ws_ypixel
+
+    @property
+    def pixel_width(self):
+        """
+        Read-only property: Width of terminal (in pixels).
+
+        :rtype: int
+        """
+        return self._height_and_width().ws_xpixel
+
     @staticmethod
     def _winsize(fd):
         """
@@ -408,6 +443,7 @@ class Terminal(object):
         :arg int fd: file descriptor queries for its window size.
         :raises IOError: the file descriptor ``fd`` is not a terminal.
         :rtype: WINSZ
+        :returns: named tuple describing size of the terminal
 
         WINSZ is a :class:`collections.namedtuple` instance, whose structure
         directly maps to the return value of the :const:`termios.TIOCGWINSZ`
@@ -419,6 +455,7 @@ class Terminal(object):
             - ``ws_ypixel``: height of terminal by pixels (not accurate).
         """
         if HAS_TTY:
+            # pylint: disable=protected-access
             data = fcntl.ioctl(fd, termios.TIOCGWINSZ, WINSZ._BUF)
             return WINSZ(*struct.unpack(WINSZ._FMT, data))
         return WINSZ(ws_row=25, ws_col=80, ws_xpixel=0, ws_ypixel=0)
@@ -434,21 +471,25 @@ class Terminal(object):
         is the default text mode of IBM PC compatibles.
 
         :rtype: WINSZ
+        :returns: Named tuple specifying the terminal size
 
         WINSZ is a :class:`collections.namedtuple` instance, whose structure
         directly maps to the return value of the :const:`termios.TIOCGWINSZ`
         ioctl return value. The return parameters are:
 
-            - ``ws_row``: width of terminal by its number of character cells.
-            - ``ws_col``: height of terminal by its number of character cells.
+            - ``ws_row``: height of terminal by its number of cell rows.
+            - ``ws_col``: width of terminal by its number of cell columns.
             - ``ws_xpixel``: width of terminal by pixels (not accurate).
             - ``ws_ypixel``: height of terminal by pixels (not accurate).
+
+            .. note:: the peculiar (height, width, width, height) order, which
+               matches the return order of TIOCGWINSZ!
         """
         for fd in (self._init_descriptor, sys.__stdout__):
             try:
                 if fd is not None:
                     return self._winsize(fd)
-            except (IOError, OSError, ValueError, TypeError):
+            except (IOError, OSError, ValueError, TypeError):  # pylint: disable=overlapping-except
                 pass
 
         return WINSZ(ws_row=int(os.getenv('LINES', '25')),
@@ -461,14 +502,22 @@ class Terminal(object):
         """
         Context manager for temporarily moving the cursor.
 
-        Move the cursor to a certain position on entry, let you print stuff
-        there, then return the cursor to its original position::
+        :arg int x: horizontal position, from left, *0*, to right edge of screen, *self.width - 1*.
+        :arg int y: vertical position, from top, *0*, to bottom of screen, *self.height - 1*.
+        :return: a context manager.
+        :rtype: Iterator
+
+        Move the cursor to a certain position on entry, do any kind of I/O, and upon exit
+        let you print stuff there, then return the cursor to its original position:
+
+
+        .. code-block:: python
 
             term = Terminal()
-            with term.location(2, 5):
-                for x in xrange(10):
-                    print('I can do it %i times!' % x)
-            print('We're back to the original location.')
+            with term.location(y=0, x=0):
+                for row_num in range(term.height-1):
+                    print('Row #{row_num}')
+            print(term.clear_eol + 'Back to original location.')
 
         Specify ``x`` to move to a certain column, ``y`` to move to a certain
         row, both, or neither. If you specify neither, only the saving and
@@ -476,9 +525,11 @@ class Terminal(object):
         simply want to restore your place after doing some manual cursor
         movement.
 
-        .. note:: The store- and restore-cursor capabilities used internally
-            provide no stack. This means that :meth:`location` calls cannot be
-            nested: only one should be entered at a time.
+        Calls cannot be nested: only one should be entered at a time.
+
+        .. note:: The argument order *(x, y)* differs from the return value order *(y, x)*
+            of :meth:`get_location`, or argument order *(y, x)* of :meth:`move`. This is
+            for API Compaibility with the blessings library, sorry for the trouble!
         """
         # pylint: disable=invalid-name
         #         Invalid argument name "x"
@@ -503,36 +554,40 @@ class Terminal(object):
         r"""
         Return tuple (row, column) of cursor position.
 
-        :arg float timeout: Return after time elapsed in seconds with value
-            ``(-1, -1)`` indicating that the remote end did not respond.
+        :arg float timeout: Return after time elapsed in seconds with value ``(-1, -1)`` indicating
+            that the remote end did not respond.
         :rtype: tuple
-        :returns: cursor position as tuple in form of (row, column).
+        :returns: cursor position as tuple in form of ``(y, x)``.  When a timeout is specified,
+            always ensure the return value is checked for ``(-1, -1)``.
 
-        The location of the cursor is determined by emitting the ``u7``
-        terminal capability, or VT100 `Query Cursor Position
-        <http://www.termsys.demon.co.uk/vtansi.htm#status>`_ when such
-        capability is undefined, which elicits a response from a reply string
-        described by capability ``u6``, or again VT100's definition of
-        ``\x1b[%i%d;%dR`` when undefined.
+        The location of the cursor is determined by emitting the ``u7`` terminal capability, or
+        VT100 `Query Cursor Position <http://www.termsys.demon.co.uk/vtansi.htm#status>`_ when such
+        capability is undefined, which elicits a response from a reply string described by
+        capability ``u6``, or again VT100's definition of ``\x1b[%i%d;%dR`` when undefined.
 
-        The ``(row, col)`` return value matches the parameter order of the
-        ``move`` capability, so that the following sequence should cause the
-        cursor to not move at all::
+        The ``(y, x)`` return value matches the parameter order of the :meth:`move_xy` capability.
+        The following sequence should cause the cursor to not move at all::
 
             >>> term = Terminal()
-            >>> term.move(*term.get_location()))
+            >>> term.move_yx(*term.get_location()))
+
+        And the following should assert True with a terminal:
+
+            >>> term = Terminal()
+            >>> given_y, given_x = 10, 20
+            >>> with term.location(y=given_y, x=given_x):
+            ...     result_y, result_x = term.get_location()
+            ...
+            >>> assert given_x == result_x, (given_x, result_x)
+            >>> assert given_y == result_y, (given_y, result_y)
 
-        .. warning:: You might first test that a terminal is capable of
-           informing you of its location, while using a timeout, before
-           later calling.  When a timeout is specified, always ensure the
-           return value is conditionally checked for ``(-1, -1)``.
         """
         # Local lines attached by termios and remote login protocols such as
         # ssh and telnet both provide a means to determine the window
         # dimensions of a connected client, but **no means to determine the
         # location of the cursor**.
         #
-        # from http://invisible-island.net/ncurses/terminfo.src.html,
+        # from https://invisible-island.net/ncurses/terminfo.src.html,
         #
         # > The System V Release 4 and XPG4 terminfo format defines ten string
         # > capabilities for use by applications, <u0>...<u9>.   In this file,
@@ -544,6 +599,7 @@ class Terminal(object):
         # >  u7   cursor position request (equiv. to VT100/ANSI/ECMA-48 DSR 6)
         # >  u6   cursor position report (equiv. to ANSI/ECMA-48 CPR)
         query_str = self.u7 or u'\x1b[6n'
+        response_str = getattr(self, self.caps['cursor_report'].attribute) or u'\x1b[%i%d;%dR'
 
         # determine response format as a regular expression
         response_re = self.caps['cursor_report'].re_compiled
@@ -558,7 +614,7 @@ class Terminal(object):
         try:
             if self._line_buffered:
                 ctx = self.cbreak()
-                ctx.__enter__()
+                ctx.__enter__()  # pylint: disable=no-member
 
             # emit the 'query cursor position' sequence,
             self.stream.write(query_str)
@@ -587,14 +643,14 @@ class Terminal(object):
                 # If the string contains the sequence %i, it is taken as an
                 # instruction to decrement each value after reading it (this is
                 # the inverse sense from the cup string).
-                if '%i' in self.cursor_report:
+                if u'%i' in response_str:
                     row -= 1
                     col -= 1
                 return row, col
 
         finally:
             if ctx is not None:
-                ctx.__exit__(None, None, None)
+                ctx.__exit__(None, None, None)  # pylint: disable=no-member
 
         # We chose to return an illegal value rather than an exception,
         # favoring that users author function filters, such as max(0, y),
@@ -649,9 +705,10 @@ class Terminal(object):
         """
         A callable string that moves the cursor to the given ``(x, y)`` screen coordinates.
 
-        :arg int x: horizontal position.
-        :arg int y: vertical position.
+        :arg int x: horizontal position, from left, *0*, to right edge of screen, *self.width - 1*.
+        :arg int y: vertical position, from top, *0*, to bottom of screen, *self.height - 1*.
         :rtype: ParameterizingString
+        :returns: Callable string that moves the cursor to the given coordinates
         """
         # this is just a convenience alias to the built-in, but hidden 'move'
         # attribute -- we encourage folks to use only (x, y) positional
@@ -663,9 +720,10 @@ class Terminal(object):
         """
         A callable string that moves the cursor to the given ``(y, x)`` screen coordinates.
 
-        :arg int x: horizontal position.
-        :arg int y: vertical position.
+        :arg int y: vertical position, from top, *0*, to bottom of screen, *self.height - 1*.
+        :arg int x: horizontal position, from left, *0*, to right edge of screen, *self.width - 1*.
         :rtype: ParameterizingString
+        :returns: Callable string that moves the cursor to the given coordinates
         """
         return self.move(y, x)
 
@@ -694,7 +752,6 @@ class Terminal(object):
         """
         A callable string that sets the foreground color.
 
-        :arg int num: The foreground color index.
         :rtype: ParameterizingString
 
         The capability is unparameterized until called and passed a number, at which point it
@@ -710,6 +767,18 @@ class Terminal(object):
                                     self.normal, 'color')
 
     def color_rgb(self, red, green, blue):
+        """
+        Provides callable formatting string to set foreground color to the specified RGB color.
+
+        :arg int red: RGB value of Red.
+        :arg int green: RGB value of Green.
+        :arg int blue: RGB value of Blue.
+        :rtype: FormattingString
+        :returns: Callable string that sets the foreground color
+
+        If the terminal does not support RGB color, the nearest supported
+        color will be determined using :py:attr:`color_distance_algorithm`.
+        """
         if self.number_of_colors == 1 << 24:
             # "truecolor" 24-bit
             fmt_attr = u'\x1b[38;2;{0};{1};{2}m'.format(red, green, blue)
@@ -724,7 +793,6 @@ class Terminal(object):
         """
         A callable capability that sets the background color.
 
-        :arg int num: The background color index.
         :rtype: ParameterizingString
         """
         if not self.does_styling:
@@ -733,6 +801,18 @@ class Terminal(object):
                                     self.normal, 'on_color')
 
     def on_color_rgb(self, red, green, blue):
+        """
+        Provides callable formatting string to set background color to the specified RGB color.
+
+        :arg int red: RGB value of Red.
+        :arg int green: RGB value of Green.
+        :arg int blue: RGB value of Blue.
+        :rtype: FormattingString
+        :returns: Callable string that sets the foreground color
+
+        If the terminal does not support RGB color, the nearest supported
+        color will be determined using :py:attr:`color_distance_algorithm`.
+        """
         if self.number_of_colors == 1 << 24:
             fmt_attr = u'\x1b[48;2;{0};{1};{2}m'.format(red, green, blue)
             return FormattingString(fmt_attr, self.normal)
@@ -740,6 +820,26 @@ class Terminal(object):
         color_idx = self.rgb_downconvert(red, green, blue)
         return FormattingString(self._background_color(color_idx), self.normal)
 
+    def formatter(self, value):
+        """
+        Provides callable formatting string to set color and other text formatting options.
+
+        :arg str value: Sugary, ordinary, or compound formatted terminal capability,
+            such as "red_on_white", "normal", "red", or "bold_on_black".
+        :rtype: :class:`FormattingString` or :class:`NullCallableString`
+        :returns: Callable string that sets color and other text formatting options
+
+        Calling ``term.formatter('bold_on_red')`` is equivalent to ``term.bold_on_red``, but a
+        string that is not a valid text formatter will return a :class:`NullCallableString`.
+        This is intended to allow validation of text formatters without the possibility of
+        inadvertently returning another terminal capability.
+        """
+        formatters = split_compound(value)
+        if all((fmt in COLORS or fmt in COMPOUNDABLES) for fmt in formatters):
+            return getattr(self, value)
+
+        return NullCallableString()
+
     def rgb_downconvert(self, red, green, blue):
         """
         Translate an RGB color to a color code of the terminal's color depth.
@@ -748,6 +848,7 @@ class Terminal(object):
         :arg int green: RGB value of Green (0-255).
         :arg int blue: RGB value of Blue (0-255).
         :rtype: int
+        :returns: Color code of downconverted RGB color
         """
         # Though pre-computing all 1 << 24 options is memory-intensive, a pre-computed
         # "k-d tree" of 256 (x,y,z) vectors of a colorspace in 3 dimensions, such as a
@@ -783,6 +884,32 @@ class Terminal(object):
         self._normal = resolve_capability(self, 'normal')
         return self._normal
 
+    def link(self, url, text, url_id=''):
+        """
+        Display ``text`` that when touched or clicked, navigates to ``url``.
+
+        Optional ``url_id`` may be specified, so that non-adjacent cells can reference a single
+        target, all cells painted with the same "id" will highlight on hover, rather than any
+        individual one, as described in "Hovering and underlining the id parameter" of gist
+        https://gist.github.com/egmontkob/eb114294efbcd5adb1944c9f3cb5feda.
+
+        :param str url: Hyperlink URL.
+        :param str text: Clickable text.
+        :param str url_id: Optional 'id'.
+        :rtype: str
+        :returns: String of ``text`` as a hyperlink to ``url``.
+        """
+        assert len(url) < 2000, (len(url), url)
+        if url_id:
+            assert len(str(url_id)) < 250, (len(str(url_id)), url_id)
+            params = 'id={0}'.format(url_id)
+        else:
+            params = ''
+        if not self.does_styling:
+            return text
+        return ('\x1b]8;{0};{1}\x1b\\{2}'
+                '\x1b]8;;\x1b\\'.format(params, url, text))
+
     @property
     def stream(self):
         """
@@ -859,6 +986,7 @@ class Terminal(object):
             unspecified, the whole width of the terminal is filled.
         :arg str fillchar: String for padding the right of ``text``
         :rtype: str
+        :returns: String of ``text``, left-aligned by ``width``.
         """
         # Left justification is different from left alignment, but we continue
         # the vocabulary error of the str method for polymorphism.
@@ -875,6 +1003,7 @@ class Terminal(object):
             unspecified, the whole width of the terminal is used.
         :arg str fillchar: String for padding the left of ``text``
         :rtype: str
+        :returns: String of ``text``, right-aligned by ``width``.
         """
         if width is None:
             width = self.width
@@ -889,6 +1018,7 @@ class Terminal(object):
             unspecified, the whole width of the terminal is used.
         :arg str fillchar: String for padding the left and right of ``text``
         :rtype: str
+        :returns: String of ``text``, centered by ``width``
         """
         if width is None:
             width = self.width
@@ -921,6 +1051,7 @@ class Terminal(object):
         Return ``text`` without sequences and leading or trailing whitespace.
 
         :rtype: str
+        :returns: Text with leading and trailing whitespace removed
 
         >>> term.strip(u' \x1b[0;3m xyz ')
         u'xyz'
@@ -932,6 +1063,7 @@ class Terminal(object):
         Return ``text`` without terminal sequences or trailing whitespace.
 
         :rtype: str
+        :returns: Text with terminal sequences and trailing whitespace removed
 
         >>> term.rstrip(u' \x1b[0;3m xyz ')
         u'  xyz'
@@ -943,6 +1075,7 @@ class Terminal(object):
         Return ``text`` without terminal sequences or leading whitespace.
 
         :rtype: str
+        :returns: Text with terminal sequences and leading whitespace removed
 
         >>> term.lstrip(u' \x1b[0;3m xyz ')
         u'xyz '
@@ -954,6 +1087,7 @@ class Terminal(object):
         Return ``text`` stripped of only its terminal sequences.
 
         :rtype: str
+        :returns: Text with terminal sequences removed
 
         >>> term.strip_seqs(u'\x1b[0;3mxyz')
         u'xyz'
@@ -966,18 +1100,33 @@ class Terminal(object):
         """
         return Sequence(text, self).strip_seqs()
 
-    def split_seqs(self, text, **kwds):
+    def split_seqs(self, text, maxsplit=0):
         r"""
         Return ``text`` split by individual character elements and sequences.
 
-        :arg kwds: remaining keyword arguments for :func:`re.split`.
+        :arg str text: String containing sequences
+        :arg int maxsplit: When maxsplit is nonzero, at most maxsplit splits
+            occur, and the remainder of the string is returned as the final element
+            of the list (same meaning is argument for :func:`re.split`).
         :rtype: list[str]
+        :returns: List of sequences and individual characters
 
         >>> term.split_seqs(term.underline(u'xyz'))
         ['\x1b[4m', 'x', 'y', 'z', '\x1b(B', '\x1b[m']
+
+        >>> term.split_seqs(term.underline(u'xyz'), 1)
+        ['\x1b[4m', r'xyz\x1b(B\x1b[m']
         """
         pattern = self._caps_unnamed_any
-        return list(filter(None, re.split(pattern, text, **kwds)))
+        result = []
+        for idx, match in enumerate(re.finditer(pattern, text)):
+            result.append(match.group())
+            if maxsplit and idx == maxsplit:
+                remaining = text[match.end():]
+                if remaining:
+                    result[-1] += remaining
+                break
+        return result
 
     def wrap(self, text, width=None, **kwargs):
         """
@@ -989,17 +1138,19 @@ class Terminal(object):
             :func:`string.expandtabs`.
         :arg int width: Unlike :func:`textwrap.wrap`, ``width`` will
             default to the width of the attached terminal.
+        :arg kwargs: See :py:class:`textwrap.TextWrapper`
         :rtype: list
+        :returns: List of wrapped lines
 
         See :class:`textwrap.TextWrapper` for keyword arguments that can
         customize wrapping behaviour.
         """
         width = self.width if width is None else width
+        wrapper = SequenceTextWrapper(width=width, term=self, **kwargs)
         lines = []
         for line in text.splitlines():
             lines.extend(
-                (_linewrap for _linewrap in SequenceTextWrapper(
-                    width=width, term=self, **kwargs).wrap(line))
+                (_linewrap for _linewrap in wrapper.wrap(line))
                 if line.strip() else (u'',))
 
         return lines
@@ -1028,7 +1179,7 @@ class Terminal(object):
         """
         Buffer input data to be discovered by next call to :meth:`~.inkey`.
 
-        :arg str ucs: String to be buffered as keyboard input.
+        :arg str text: String to be buffered as keyboard input.
         """
         self._keyboard_buf.extendleft(text)
 
