@@ -1,4 +1,4 @@
-# encoding: utf-8
+# -*- coding: utf-8 -*-
 # std imports
 import os
 import sys
@@ -10,8 +10,8 @@ import itertools
 import six
 import pytest
 
-from .accessories import (  # isort:skip
-    TestTerminal, as_subprocess, all_terms, many_lines, many_columns)
+# local
+from .accessories import TestTerminal, as_subprocess
 
 if platform.system() != 'Windows':
     import fcntl
@@ -60,6 +60,11 @@ def test_sequence_length(all_terms):
     @as_subprocess
     def child(kind):
         term = TestTerminal(kind=kind, force_styling=True)
+
+        # Make sure to test with 24-bit color on at least one terminal
+        if kind == 'xterm':
+            term.number_of_colors = 1 << 24
+
         # Create a list of ascii characters, to be separated
         # by word, to be zipped up with a cycling list of
         # terminal sequences. Then, compare the length of
@@ -74,6 +79,12 @@ def test_sequence_length(all_terms):
             assert (term.length(term.bold(u'x')) == 1)
             assert (term.length(term.bold_red) == 0)
             assert (term.length(term.bold_red(u'x')) == 1)
+            assert (term.length(term.bold_on_red) == 0)
+            assert (term.length(term.bold_on_red(u'x')) == 1)
+            assert (term.length(term.bold_olivedrab4) == 0)
+            assert (term.length(term.bold_olivedrab4(u'x')) == 1)
+            assert (term.length(term.bold_on_olivedrab4) == 0)
+            assert (term.length(term.bold_on_olivedrab4(u'x')) == 1)
             assert (term.strip(term.bold) == u'')
             assert (term.rstrip(term.bold) == u'')
             assert (term.lstrip(term.bold) == u'')
@@ -84,60 +95,150 @@ def test_sequence_length(all_terms):
             assert (term.strip(term.bold_red) == u'')
             assert (term.rstrip(term.bold_red) == u'')
             assert (term.lstrip(term.bold_red) == u'')
+            assert (term.strip(term.bold_on_red) == u'')
+            assert (term.rstrip(term.bold_on_red) == u'')
+            assert (term.lstrip(term.bold_on_red) == u'')
+            assert (term.strip(term.bold_olivedrab4) == u'')
+            assert (term.rstrip(term.bold_olivedrab4) == u'')
+            assert (term.lstrip(term.bold_olivedrab4) == u'')
+            assert (term.strip(term.bold_on_olivedrab4) == u'')
+            assert (term.rstrip(term.bold_on_olivedrab4) == u'')
+            assert (term.lstrip(term.bold_on_olivedrab4) == u'')
             assert (term.strip(term.bold_red(u'  x  ')) == u'x')
             assert (term.rstrip(term.bold_red(u'  x  ')) == u'  x')
             assert (term.lstrip(term.bold_red(u'  x  ')) == u'x  ')
+            assert (term.strip(term.bold_on_red(u'  x  ')) == u'x')
+            assert (term.rstrip(term.bold_on_red(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.bold_on_red(u'  x  ')) == u'x  ')
+            assert (term.strip(term.bold_olivedrab4(u'  x  ')) == u'x')
+            assert (term.rstrip(term.bold_olivedrab4(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.bold_olivedrab4(u'  x  ')) == u'x  ')
+            assert (term.strip(term.bold_on_olivedrab4(u'  x  ')) == u'x')
+            assert (term.rstrip(term.bold_on_olivedrab4(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.bold_on_olivedrab4(u'  x  ')) == u'x  ')
             assert (term.strip_seqs(term.bold) == u'')
             assert (term.strip_seqs(term.bold(u'  x  ')) == u'  x  ')
             assert (term.strip_seqs(term.bold_red) == u'')
             assert (term.strip_seqs(term.bold_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.bold_on_red) == u'')
+            assert (term.strip_seqs(term.bold_on_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.bold_olivedrab4) == u'')
+            assert (term.strip_seqs(term.bold_olivedrab4(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.bold_on_olivedrab4) == u'')
+            assert (term.strip_seqs(term.bold_on_olivedrab4(u'  x  ')) == u'  x  ')
 
         if term.underline:
             assert (term.length(term.underline) == 0)
             assert (term.length(term.underline(u'x')) == 1)
             assert (term.length(term.underline_red) == 0)
             assert (term.length(term.underline_red(u'x')) == 1)
+            assert (term.length(term.underline_on_red) == 0)
+            assert (term.length(term.underline_on_red(u'x')) == 1)
+            assert (term.length(term.underline_olivedrab4) == 0)
+            assert (term.length(term.underline_olivedrab4(u'x')) == 1)
+            assert (term.length(term.underline_on_olivedrab4) == 0)
+            assert (term.length(term.underline_on_olivedrab4(u'x')) == 1)
             assert (term.strip(term.underline) == u'')
             assert (term.strip(term.underline(u'  x  ')) == u'x')
             assert (term.strip(term.underline_red) == u'')
             assert (term.strip(term.underline_red(u'  x  ')) == u'x')
             assert (term.rstrip(term.underline_red(u'  x  ')) == u'  x')
             assert (term.lstrip(term.underline_red(u'  x  ')) == u'x  ')
+            assert (term.strip(term.underline_on_red) == u'')
+            assert (term.strip(term.underline_on_red(u'  x  ')) == u'x')
+            assert (term.rstrip(term.underline_on_red(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.underline_on_red(u'  x  ')) == u'x  ')
+            assert (term.strip(term.underline_olivedrab4) == u'')
+            assert (term.strip(term.underline_olivedrab4(u'  x  ')) == u'x')
+            assert (term.rstrip(term.underline_olivedrab4(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.underline_olivedrab4(u'  x  ')) == u'x  ')
+            assert (term.strip(term.underline_on_olivedrab4) == u'')
+            assert (term.strip(term.underline_on_olivedrab4(u'  x  ')) == u'x')
+            assert (term.rstrip(term.underline_on_olivedrab4(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.underline_on_olivedrab4(u'  x  ')) == u'x  ')
             assert (term.strip_seqs(term.underline) == u'')
             assert (term.strip_seqs(term.underline(u'  x  ')) == u'  x  ')
             assert (term.strip_seqs(term.underline_red) == u'')
             assert (term.strip_seqs(term.underline_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.underline_on_red) == u'')
+            assert (term.strip_seqs(term.underline_on_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.underline_olivedrab4) == u'')
+            assert (term.strip_seqs(term.underline_olivedrab4(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.underline_on_olivedrab4) == u'')
+            assert (term.strip_seqs(term.underline_on_olivedrab4(u'  x  ')) == u'  x  ')
 
         if term.reverse:
             assert (term.length(term.reverse) == 0)
             assert (term.length(term.reverse(u'x')) == 1)
             assert (term.length(term.reverse_red) == 0)
             assert (term.length(term.reverse_red(u'x')) == 1)
+            assert (term.length(term.reverse_on_red) == 0)
+            assert (term.length(term.reverse_on_red(u'x')) == 1)
+            assert (term.length(term.reverse_olivedrab4) == 0)
+            assert (term.length(term.reverse_olivedrab4(u'x')) == 1)
+            assert (term.length(term.reverse_on_olivedrab4) == 0)
+            assert (term.length(term.reverse_on_olivedrab4(u'x')) == 1)
             assert (term.strip(term.reverse) == u'')
             assert (term.strip(term.reverse(u'  x  ')) == u'x')
             assert (term.strip(term.reverse_red) == u'')
             assert (term.strip(term.reverse_red(u'  x  ')) == u'x')
             assert (term.rstrip(term.reverse_red(u'  x  ')) == u'  x')
             assert (term.lstrip(term.reverse_red(u'  x  ')) == u'x  ')
+            assert (term.strip(term.reverse_on_red) == u'')
+            assert (term.strip(term.reverse_on_red(u'  x  ')) == u'x')
+            assert (term.rstrip(term.reverse_on_red(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.reverse_on_red(u'  x  ')) == u'x  ')
+            assert (term.strip(term.reverse_olivedrab4) == u'')
+            assert (term.strip(term.reverse_olivedrab4(u'  x  ')) == u'x')
+            assert (term.rstrip(term.reverse_olivedrab4(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.reverse_olivedrab4(u'  x  ')) == u'x  ')
+            assert (term.strip(term.reverse_on_olivedrab4) == u'')
+            assert (term.strip(term.reverse_on_olivedrab4(u'  x  ')) == u'x')
+            assert (term.rstrip(term.reverse_on_olivedrab4(u'  x  ')) == u'  x')
+            assert (term.lstrip(term.reverse_on_olivedrab4(u'  x  ')) == u'x  ')
             assert (term.strip_seqs(term.reverse) == u'')
             assert (term.strip_seqs(term.reverse(u'  x  ')) == u'  x  ')
             assert (term.strip_seqs(term.reverse_red) == u'')
             assert (term.strip_seqs(term.reverse_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.reverse_on_red) == u'')
+            assert (term.strip_seqs(term.reverse_on_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.reverse_olivedrab4) == u'')
+            assert (term.strip_seqs(term.reverse_olivedrab4(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.reverse_on_olivedrab4) == u'')
+            assert (term.strip_seqs(term.reverse_on_olivedrab4(u'  x  ')) == u'  x  ')
 
         if term.blink:
             assert (term.length(term.blink) == 0)
             assert (term.length(term.blink(u'x')) == 1)
             assert (term.length(term.blink_red) == 0)
             assert (term.length(term.blink_red(u'x')) == 1)
+            assert (term.length(term.blink_on_red) == 0)
+            assert (term.length(term.blink_on_red(u'x')) == 1)
+            assert (term.length(term.blink_olivedrab4) == 0)
+            assert (term.length(term.blink_olivedrab4(u'x')) == 1)
+            assert (term.length(term.blink_on_olivedrab4) == 0)
+            assert (term.length(term.blink_on_olivedrab4(u'x')) == 1)
             assert (term.strip(term.blink) == u'')
             assert (term.strip(term.blink(u'  x  ')) == u'x')
             assert (term.strip(term.blink(u'z  x  q'), u'zq') == u'  x  ')
             assert (term.strip(term.blink_red) == u'')
             assert (term.strip(term.blink_red(u'  x  ')) == u'x')
+            assert (term.strip(term.blink_on_red) == u'')
+            assert (term.strip(term.blink_on_red(u'  x  ')) == u'x')
+            assert (term.strip(term.blink_olivedrab4) == u'')
+            assert (term.strip(term.blink_olivedrab4(u'  x  ')) == u'x')
+            assert (term.strip(term.blink_on_olivedrab4) == u'')
+            assert (term.strip(term.blink_on_olivedrab4(u'  x  ')) == u'x')
             assert (term.strip_seqs(term.blink) == u'')
             assert (term.strip_seqs(term.blink(u'  x  ')) == u'  x  ')
             assert (term.strip_seqs(term.blink_red) == u'')
             assert (term.strip_seqs(term.blink_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.blink_on_red) == u'')
+            assert (term.strip_seqs(term.blink_on_red(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.blink_olivedrab4) == u'')
+            assert (term.strip_seqs(term.blink_olivedrab4(u'  x  ')) == u'  x  ')
+            assert (term.strip_seqs(term.blink_on_olivedrab4) == u'')
+            assert (term.strip_seqs(term.blink_on_olivedrab4(u'  x  ')) == u'  x  ')
 
         if term.home:
             assert (term.length(term.home) == 0)
@@ -223,10 +324,11 @@ def test_env_winsize():
 @pytest.mark.skipif(platform.system() == 'Windows', reason="requires fcntl")
 def test_winsize(many_lines, many_columns):
     """Test height and width is appropriately queried in a pty."""
+    pixel_width, pixel_height = 1024, 768
     @as_subprocess
     def child(lines=25, cols=80):
         # set the pty's virtual window size
-        val = struct.pack('HHHH', lines, cols, 0, 0)
+        val = struct.pack('HHHH', lines, cols, pixel_width, pixel_height)
         fcntl.ioctl(sys.__stdout__.fileno(), termios.TIOCSWINSZ, val)
         term = TestTerminal()
         winsize = term._height_and_width()
@@ -234,11 +336,13 @@ def test_winsize(many_lines, many_columns):
         assert term.height == lines
         assert winsize.ws_col == cols
         assert winsize.ws_row == lines
+        assert term.pixel_width == pixel_width
+        assert term.pixel_height == pixel_height
 
     child(lines=many_lines, cols=many_columns)
 
 
-def test_Sequence_alignment_fixed_width():
+def test_Sequence_alignment_fixed_width(all_terms):
     @as_subprocess
     def child(kind):
         term = TestTerminal(kind=kind)
@@ -259,6 +363,8 @@ def test_Sequence_alignment_fixed_width():
         assert (term.length(radjusted.strip()) == pony_len)
         assert (term.length(radjusted) == len(pony_msg.rjust(88)))
 
+    child(kind=all_terms)
+
 
 @pytest.mark.skipif(platform.system() == 'Windows', reason="requires fcntl")
 def test_Sequence_alignment(all_terms):
@@ -290,6 +396,65 @@ def test_Sequence_alignment(all_terms):
     child(kind=all_terms)
 
 
+def test_hyperlink_nostyling():
+    """Test length our of hyperlink URL's."""
+    @as_subprocess
+    def child():
+        # given,
+        term = TestTerminal(force_styling=None)
+        given_basic_url = term.link(
+            'https://blessed.readthedocs.org', 'blessed')
+        assert given_basic_url == 'blessed'
+
+    child()
+
+
+def test_basic_hyperlinks():
+    """Test length our of hyperlink URL's."""
+    @as_subprocess
+    def child():
+        # given,
+        term = TestTerminal()
+        given_basic_url = term.link(
+            'https://blessed.readthedocs.org', 'blessed')
+        # exercise,
+        split_parts = term.split_seqs(given_basic_url)
+        # verify
+        if term.does_styling:
+            assert split_parts[0] == '\x1b]8;;https://blessed.readthedocs.org\x1b\\'
+            assert term.length(split_parts[0]) == 0
+            assert ''.join(split_parts[1:8]) == 'blessed'
+            assert split_parts[8] == '\x1b]8;;\x1b\\'
+            assert len(split_parts) == 9
+        else:
+            assert ''.join(split_parts) == 'blessed'
+
+    child()
+
+
+def test_hyperlink_with_id():
+    """Test length our of hyperlink URL's with ID."""
+    @as_subprocess
+    def child():
+        # given,
+        term = TestTerminal()
+        given_advanced_urltext = term.link(
+            'https://blessed.readthedocs.org', 'blessed', '123')
+        # exercise,
+        split_parts = term.split_seqs(given_advanced_urltext)
+        # verify,
+        if term.does_styling:
+            assert split_parts[0] == '\x1b]8;id=123;https://blessed.readthedocs.org\x1b\\'
+            assert term.length(split_parts[0]) == 0
+            assert ''.join(split_parts[1:8]) == 'blessed'
+            assert split_parts[8] == '\x1b]8;;\x1b\\'
+            assert len(split_parts) == 9
+        else:
+            assert ''.join(split_parts) == 'blessed'
+
+    child()
+
+
 def test_sequence_is_movement_false(all_terms):
     """Test parser about sequences that do not move the cursor."""
     @as_subprocess
